/***
 * Fractal ADL Parser
 * Copyright (C) 2002-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Eric.Bruneton@rd.francetelecom.com
 *
 * Author: Eric Bruneton
 */

package org.objectweb.fractal.adl.arguments;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Definition;
import org.objectweb.fractal.adl.components.ComponentLoader;

/**
 * Extends {@link ComponentLoader} to take ADL arguments into account.
 */
public class ArgumentComponentLoader extends ComponentLoader {

  @Override
  protected Definition load(final List<String> loaded, final String name,
      final Map<Object, Object> context) throws ADLException {
    String n = name;
    Map<Object, Object> c = context;
    int i = name.indexOf('(');
    if (i != -1) {
      if (!name.endsWith(")")) {
        throw new ADLException("Syntax error in definition name");
      }

      n = name.substring(0, i);
      c = new HashMap<Object, Object>();
      for (final Map.Entry<Object, Object> entry : context.entrySet()) {
        if (!(entry.getKey() instanceof String)) continue;
        final String key = (String) entry.getKey();
        if (!key.startsWith(" arg ")) {
          c.put(key, entry.getValue());
        }
      }

      final String args = name.substring(i + 1, name.length() - 1);
      final StringTokenizer st = new StringTokenizer(args, ",");
      i = 0;
      while (st.hasMoreTokens()) {
        c.put(" arg " + (i++), st.nextToken());
      }
    }
    return super.load(loaded, n, c);
  }

  @Override
  public List<String> parseDefinitions(String nameList) {
    final List<String> l = new ArrayList<String>();
    int c = nameList.lastIndexOf('(');
    int p = c == -1 ? nameList.lastIndexOf(',') : nameList.lastIndexOf(',', c);
    while (p != -1) {
      l.add(0, nameList.substring(p + 1));
      nameList = nameList.substring(0, p);
      c = nameList.lastIndexOf('(');
      p = c == -1 ? nameList.lastIndexOf(',') : nameList.lastIndexOf(',', c);
    }
    l.add(0, nameList);
    return l;
  }

  @Override
  public boolean isShared(final String definition) {
    return definition.indexOf('/') != -1 && definition.indexOf('(') == -1;
  }
}
